import { IUser } from '@/containers/Auth';
import { axiosApiClient } from '../helpers/axiosApiClient';
import { createAsyncThunk, createSlice } from '@reduxjs/toolkit';
import { AxiosError, isAxiosError } from 'axios';

interface State {
  user: IUser | null;
  error: Error | null;
  loading: boolean;
}

const initialState: State = {
  user: null,
  error: null,
  loading: false,
};

export const refreshToken = createAsyncThunk('auth/refreshToken', async () => {
  return await axiosApiClient
    .get<IUser | null>('/auth/refreshToken', {
      headers: {
        Authorization: `Bearer ${localStorage.getItem('token')}`,
      },
    })
    .then((res) => res.data)
    .catch(() => {
      localStorage.setItem('token', '');
    });
});

export const register = createAsyncThunk(
  'auth/register',
  async (payload: { username: string; password: string }) => {
    return await axiosApiClient.post<IUser | null>('/auth/register', payload).then((res) => {
      localStorage.setItem('token', res.data?.accessToken || '');
      return res.data;
    });
  }
);

export const login = createAsyncThunk(
  'auth/login',
  async (payload: { username: string; password: string }, { rejectWithValue }) => {
    try {
      return await axiosApiClient.post<IUser | null>('/auth/login', payload).then((res) => {
        localStorage.setItem('token', res.data?.accessToken || '');
        return res.data;
      });
    } catch (error) {
      if (isAxiosError(error)) {
        const err: AxiosError<{ message: string }> = error;
        return rejectWithValue(err.response?.data || { error: { message: 'An error occurred' } });
      }

      throw error;
    }
  }
);

const usersSlice = createSlice({
  name: 'users',
  initialState,
  reducers: {},
  extraReducers(builder) {
    builder

      // REGISTER
      .addCase(register.fulfilled, (state, action) => {
        state.user = action.payload;
        state.loading = false;
      })
      .addCase(register.rejected, (state, action) => {
        state.loading = false;
        state.error = action.error as Error;
      })
      .addCase(register.pending, (state) => {
        state.loading = true;
      })

      // LOGIN
      .addCase(login.fulfilled, (state, action) => {
        state.user = action.payload;
        state.loading = false;
      })
      .addCase(login.rejected, (state, action) => {
        state.loading = false;
        state.error = action.payload.message;
      })
      .addCase(login.pending, (state) => {
        state.loading = true;
      })

      // REFRESH TOKEN
      .addCase(refreshToken.fulfilled, (state, action) => {
        state.user = action.payload;
        state.loading = false;
      })
      .addCase(refreshToken.rejected, (state, action) => {
        state.loading = false;
        state.error = action.payload.message;
      })
      .addCase(refreshToken.pending, (state) => {
        state.loading = true;
      });
  },
});

export default usersSlice.reducer;
